require 'shipyard_client/version'
require 'json'
require 'http'

module ShipyardClient
  class Shipyard
    attr_accessor :host, :service_key
    def initialize(host, service_key)
      @host = host
      @service_key = service_key
    end

    def list_containers
      res = HTTP.headers('X-Service-Key' => @service_key)
                .get("#{host}/containers/json")
      handle_response(res)
    end

    def create_container(image, options)
      create_template = {
        AttachStdin: false,
        CpuShares: nil,
        Env: [],
        Cmd: [],
        ExposedPorts: {},
        HostConfig: {
          Binds: [],
          Links: [],
          PortBindings: {},
          Privileged: false,
          PublishAllPorts: false,
          RestartPolicy: {
            Name: 'no'
          }
        },
        Image: '',
        Links: [],
        Memory: nil,
        Tty: true,
        Volumes: {}
      }
      create_template[:Image] = image
      create_template.merge!(options)
      res = HTTP.headers('X-Service-Key' => @service_key)
                .post("#{host}/containers/create?name=",
                      json: create_template)
      handle_response(res)
    end

    def inspect_container(id)
      res = HTTP.headers('X-Service-Key' => @service_key)
                .get("#{host}/containers/#{id}/json")
      handle_response(res)
    end

    def remove_container(id)
      res = HTTP.headers('X-Service-Key' => @service_key)
                .delete("#{host}/containers/#{id}?force=1")
      handle_response(res)
    end

    def start_container(id)
      res = HTTP.headers('X-Service-Key' => @service_key)
                .post("#{host}/containers/#{id}/start")
      handle_response(res)
    end

    def stop_container(id)
      res = HTTP.headers('X-Service-Key' => @service_key)
                .post("#{host}/containers/#{id}/stop")
      handle_response(res)
    end

    def restart_container(id)
      res = HTTP.headers('X-Service-Key' => @service_key)
                .post("#{host}/containers/#{id}/restart")
      handle_response(res)
    end

    private

    def handle_response(res)
      if res.code < 200 || res.code >= 300
        raise "FAILED with status code: #{res.code} #{res}"
      end
      # Do not parse JSON if there is no content
      res.code == 204 ? nil : JSON.load(res.to_s)
    end
  end
end
